@@ -1,5 +1,6 @@
 import time
 import json
+import sys
 
 try:
     import IPython.display
@@ -7,6 +8,13 @@ try:
 except ImportError:
     _has_IPython = False
 
+if sys.version_info >= (3, 8):
+    from importlib import metadata
+else:
+    import importlib_metadata as metadata
+
+__version__ = metadata.version('py3Dmol')
+
 #surface type constants
 VDW =1
 MS=2
@@ -22,6 +30,14 @@ def using_ipython(func):
 
     return inner
 
+def tostr(arg):
+    '''convert an argument to javascript string,
+    can handle top-level numpy/torch arrays'''
+    try:
+        return json.dumps(arg)
+    except TypeError:
+        return json.dumps(arg.tolist()) #numpy arrays and torch tensor
+        
 class view(object):
     '''A class for constructing embedded 3Dmol.js views in ipython notebooks.
        The results are completely static which means there is no need for there
@@ -35,9 +51,9 @@ class view(object):
 
        The API for the created object is exactly that for $3Dmol.GLViewer, with
        the exception that the functions all return None.
-       http://3dmol.org/doc/$3Dmol.GLViewer.html
+       http://3dmol.org/doc/GLViewer.html
     '''
-    def __init__(self,width=640,height=480,query='',viewergrid=None,data=None,style=None,linked=True,options=dict(),js='https://3dmol.org/build/3Dmol.js'):
+    def __init__(self,query='',width=640,height=480,viewergrid=None,data=None,style=None,linked=True,options=dict(),js='https://cdn.jsdelivr.net/npm/3dmol@latest/build/3Dmol-min.min.js'):
         '''Create a 3Dmol.js view.
             width -- width in pixels of container
             height -- height in pixels of container
@@ -50,7 +66,12 @@ class view(object):
         divid = "3dmolviewer_UNIQUEID"
         warnid = "3dmolwarning_UNIQUEID"
         self.uniqueid = None
-        self.startjs = '''<div id="%s"  style="position: relative; width: %dpx; height: %dpx">
+        #convert numerical width/height to pixels
+        if type(width) == int:
+            width = '%dpx'%width
+        if type(height) == int:
+            height = '%dpx'%height
+        self.startjs = '''<div id="%s"  style="position: relative; width: %s; height: %s">
         <p id="%s" style="background-color:#ffcccc;color:black">You appear to be running in JupyterLab (or JavaScript failed to load for some other reason).  You need to install the 3dmol extension: <br>
         <tt>jupyter labextension install jupyterlab_3dmol</tt></p>
         </div>\n''' % (divid,width,height,warnid)
@@ -63,11 +84,20 @@ class view(object):
         self.startjs += """
 var loadScriptAsync = function(uri){
   return new Promise((resolve, reject) => {
+    //this is to ignore the existence of requirejs amd
+    var savedexports, savedmodule;
+    if (typeof exports !== 'undefined') savedexports = exports;
+    else exports = {}
+    if (typeof module !== 'undefined') savedmodule = module;
+    else module = {}
+
     var tag = document.createElement('script');
     tag.src = uri;
     tag.async = true;
     tag.onload = () => {
-      resolve();
+        exports = savedexports;
+        module = savedmodule;
+        resolve();
     };
   var firstScriptTag = document.getElementsByTagName('script')[0];
   firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);
@@ -94,11 +124,11 @@ if(warn) {
             if len(viewergrid) != 2:
                 raise ValueError("Incorrectly formated viewergrid arguments.  Must specify rows x columns",viewergrid)
             self.startjs += "var viewergrid_UNIQUEID = null;\n";
-            self.startjs += 'viewergrid_UNIQUEID = $3Dmol.createViewerGrid($("#%s"),{rows: %d, cols: %d, control_all: %s},{backgroundColor:"white"});\n' % (divid, viewergrid[0],viewergrid[1],'true' if linked else 'false')
+            self.startjs += 'viewergrid_UNIQUEID = $3Dmol.createViewerGrid(document.getElementById("%s"),{rows: %d, cols: %d, control_all: %s},{backgroundColor:"white"});\n' % (divid, viewergrid[0],viewergrid[1],'true' if linked else 'false')
             self.startjs += "viewer_UNIQUEID = viewergrid_UNIQUEID[0][0];\n"
             self.viewergrid = viewergrid
         else:
-            self.startjs += 'viewer_UNIQUEID = $3Dmol.createViewer($("#%s"),{backgroundColor:"white"});\n' % divid
+            self.startjs += 'viewer_UNIQUEID = $3Dmol.createViewer(document.getElementById("%s"),{backgroundColor:"white"});\n' % divid
         if query:
             if viewergrid:
                 for r in range(viewergrid[0]):
@@ -142,15 +172,15 @@ if(warn) {
         '''Instantiate a new viewer window. Calling this will orphan any previously instantiated viewer windows.'''
         self.updatejs = ''
         html = self._make_html()
-        return IPython.display.publish_display_data({'application/3dmoljs_load.v0':html, 'text/html': html})
+        return IPython.display.publish_display_data({'application/3dmoljs_load.v0':html, 'text/html': html},metadata={})
 
     @using_ipython
     def insert(self, containerid):
         '''Instead of inserting into notebook here, insert html
         into existing container'''
         html = self._make_html()
-        html += '''<script>$("#%s").append($("#3dmolviewer_%s")); </script>'''%(containerid,self.uniqueid)
-        return IPython.display.publish_display_data({'application/3dmoljs_load.v0':html, 'text/html': html})
+        html += '''<script>document.getElementById("%s").append(document.getElementById("3dmolviewer_%s")); </script>'''%(containerid,self.uniqueid)
+        return IPython.display.publish_display_data({'application/3dmoljs_load.v0':html, 'text/html': html},metadata={})
 
     def _make_html(self):
         self.uniqueid = str(time.time()).replace('.','')
@@ -161,7 +191,7 @@ if(warn) {
     @using_ipython
     def _repr_html_(self):
         html = self._make_html()
-        return IPython.display.publish_display_data({'application/3dmoljs_load.v0':html, 'text/html': html})
+        return IPython.display.publish_display_data({'application/3dmoljs_load.v0':html, 'text/html': html},metadata={})
 
     @using_ipython
     def update(self):
@@ -176,7 +206,7 @@ if(warn) {
             });
             </script>''' % (self.updatejs.replace('UNIQUEID',self.uniqueid),self.uniqueid)
         self.updatejs = ''
-        return IPython.display.publish_display_data({'application/3dmoljs_load.v0':script, 'text/html': script})
+        return IPython.display.publish_display_data({'application/3dmoljs_load.v0':script, 'text/html': script},metadata={})
 
     @using_ipython
     def png(self):
@@ -186,16 +216,28 @@ if(warn) {
         script = '''<img id="img_{0}">
             <script>
             var png = viewer_{0}.pngURI()
-            $('#img_{0}').attr('src', png)
+            document.getElementById('img_{0}').src = png;
             </script>'''.format(self.uniqueid)
-        return IPython.display.publish_display_data({'application/3dmoljs_load.v0':script, 'text/html': script})
-
+        return IPython.display.publish_display_data({'application/3dmoljs_load.v0':script, 'text/html': script},metadata={})
+    
+    @using_ipython
+    def apng(self, nframes=1):
+        '''output animated image of viewer, which must already be instantiated. nframes changes will be captured.'''
+        if not self.uniqueid:
+            raise AssertionError('Must instantiate viewer before generating animated image.')
+        script = '''<img id="img_{0}">
+            <script>
+            viewer_{0}.apngURI({1}).then(png => {{
+            document.getElementById('img_{0}').src = png; }});
+            </script>'''.format(self.uniqueid,nframes)
+        return IPython.display.publish_display_data({'application/3dmoljs_load.v0':script, 'text/html': script},metadata={})    
+        
     class model(object):
       '''Wrapper for referencing a model within a viewer'''
       def __init__(self, viewer, accesscmd):
         self.accesscmd = accesscmd
         self.viewer = viewer
-
+        
       def __getattr__(self,name):
         '''auto-instantiate javascript calls through model'''
         if name.startswith('_'): #object to ipython canary functions
@@ -204,7 +246,7 @@ if(warn) {
         def makejs(*args,**kwargs):
           cmd = '\t%s.%s(' % (self.accesscmd,name);
           for arg in args:
-              cmd += '%s,' % json.dumps(arg)
+              cmd += '%s,' % tostr(arg)
           cmd = cmd.rstrip(',')
           cmd += ');\n';
 
@@ -221,7 +263,7 @@ if(warn) {
               raise ValueError("Incorrectly formated viewer argument.  Must specify row and column",self.viewergrid)
           cmd = '\tviewergrid_UNIQUEID[%d][%d].getModel(' % (coords[0],coords[1]);
           for arg in args:
-              cmd += '%s,' % json.dumps(arg)
+              cmd += '%s,' % tostr(arg)
           cmd = cmd.rstrip(',')
           cmd += ')';
         else:
@@ -229,16 +271,16 @@ if(warn) {
       else:
         cmd = '\tviewer_UNIQUEID.getModel(';
         for arg in args:
-            cmd += '%s,' % json.dumps(arg)
+            cmd += '%s,' % tostr(arg)
         cmd = cmd.rstrip(',')
         cmd += ')';
       return self.model(self,cmd)
 
     def __getattr__(self,name):
         '''auto-instantiate javascript calls based on whatever the user provided'''
-        if name.startswith('_'): #object to ipython canary functions
+        if name.startswith('_') or name == 'getdoc': #object to ipython canary functions
             raise AttributeError("%r object has no attribute %r" %  (self.__class__, name))
-
+            
         def makejs(*args,**kwargs):
             if self.viewergrid:
                 if kwargs and 'viewer' in kwargs:
@@ -247,7 +289,7 @@ if(warn) {
                         raise ValueError("Incorrectly formated viewer argument.  Must specify row and column",self.viewergrid)
                     cmd = '\tviewergrid_UNIQUEID[%d][%d].%s(' % (coords[0],coords[1],name);
                     for arg in args:
-                        cmd += '%s,' % json.dumps(arg)
+                        cmd += '%s,' % tostr(arg)
                     cmd = cmd.rstrip(',')
                     cmd += ');\n';
                 else: #apply to every viewer
@@ -256,13 +298,13 @@ if(warn) {
                         for c in range(self.viewergrid[1]):
                             cmd += '\tviewergrid_UNIQUEID[%d][%d].%s(' % (r,c,name);
                             for arg in args:
-                                cmd += '%s,' % json.dumps(arg)
+                                cmd += '%s,' % tostr(arg)
                             cmd = cmd.rstrip(',')
                             cmd += ');\n';
             else:
                 cmd = '\tviewer_UNIQUEID.%s(' % name;
                 for arg in args:
-                    cmd += '%s,' % json.dumps(arg)
+                    cmd += '%s,' % tostr(arg)
                 cmd = cmd.rstrip(',')
                 cmd += ');\n';
 
